//-----------------------------------------------------------------------------
/*
Programmed by Olivier Sangala
http://olivier.sangala.free.fr
olivier_sangala@hotmail.com
*/
//-----------------------------------------------------------------------------

#include <tigcclib.h>

//-----------------------------------------------------------------------------

#define QUEENS_COUNT (8)
#define SOLUTIONS_MAX (92)

//-----------------------------------------------------------------------------

typedef struct
{
	char x;
	char y;
}
TPoint;

//-----------------------------------------------------------------------------

int IsSafePlace(int q);
void ChoosePlace(int q);
void DrawScreen(void);

//-----------------------------------------------------------------------------

TPoint SolutionsList[SOLUTIONS_MAX][QUEENS_COUNT];
TPoint Q[QUEENS_COUNT];

int SolutionsIndex;
int SolutionsCount;

char*Plane0;


//-----------------------------------------------------------------------------

unsigned short TileData_OFF[16]=
{
	0b1111111111110000,
	0b1000000000000000,
	0b1000000000000000,
	0b1000000000000000,
	0b1000000000000000,
	0b1000000000000000,
	0b1000000000000000,
	0b1000000000000000,
	0b1000000000000000,
	0b1000000000000000,
	0b1000000000000000,
	0b1000000000000000,
	0b0000000000000000,
	0b0000000000000000,
	0b0000000000000000,
	0b0000000000000000,
};

unsigned short TileData_ON[16]=
{
	0b1111111111110000,
	0b1000000000000000,
	0b1011111111100000,
	0b1011111111100000,
	0b1011111111100000,
	0b1011111111100000,
	0b1011111111100000,
	0b1011111111100000,
	0b1011111111100000,
	0b1011111111100000,
	0b1011111111100000,
	0b1000000000000000,
	0b0000000000000000,
	0b0000000000000000,
	0b0000000000000000,
	0b0000000000000000,
};

//-----------------------------------------------------------------------------

int IsSafePlace(int q)
{
	if (!q)
	return 1;
	
	int a;
	
	// SAME ROW ?
	for (a=0;a<q;a++)
	if (Q[a].y==Q[q].y)
	return 0;
	
	// SAME COLUMN ?
	for (a=0;a<q;a++)
	if (Q[a].x==Q[q].x)
	return 0;
	
	// SAME DIAGONAL ?
	for (a=0;a<q;a++)
	if (abs(Q[a].x-Q[q].x)==abs(Q[a].y-Q[q].y))
	return 0;
	
	return 1;
}

//-----------------------------------------------------------------------------

void ChoosePlace(int q)
{
	
	int a;
	
	for (a=0;a<QUEENS_COUNT;a++)
	{
		Q[q].y=a;
		if (IsSafePlace(q))
		{
			// IF SOLUTION FOUND
			if (q+1==QUEENS_COUNT)
			{
				// Store Solution
				memcpy(SolutionsList[SolutionsCount++],Q,16);
			}
			else
			ChoosePlace(q+1);
		}
	}
}


//-----------------------------------------------------------------------------

void DrawScreen(void)
{

	int a;
	
	char buffer[50];
	
	// Get Solution At Index
	memcpy(Q,&SolutionsList[SolutionsIndex][0],16);
	
	memcpy(LCD_MEM,Plane0,LCD_SIZE);
	
	for (a=0;a<QUEENS_COUNT;a++)
	Sprite16(12*a,12*Q[a].y,16,TileData_ON,LCD_MEM,SPRT_OR);
	
	
	// Solution Counter
	sprintf(buffer,"<%02d/%02d>",SolutionsIndex+1,SolutionsCount);
	DrawStr(100,0,buffer,A_NORMAL);
	
	// Solution Value
	for (a=0;a<QUEENS_COUNT;a++)
	buffer[a]='0'+Q[a].y;
	
	buffer[QUEENS_COUNT]=0;
	DrawStr(100,10,buffer,A_NORMAL);

}
//-----------------------------------------------------------------------------

// Main Function
void _main(void)
{
	
	Plane0=malloc(LCD_SIZE);

	clrscr();
	FontSetSys(0);
	printf("Queens 1.0 \n");
	printf("Programmed by Olivier Sangala \n");
	printf("http://olivier.sangala.free.fr \n");
	printf("olivier_sangala@hotmail.com \n");
	FontSetSys(1);
	ngetchx();
	
	
	int a;
	for (a=0;a<QUEENS_COUNT;a++)
	Q[a].x=a;
	
	// Make & Save BG Screen
	
	PortSet(Plane0,239,127);
	ClrScr();
	
	int x,y;
	for (x=0;x<QUEENS_COUNT;x++)
	for (y=0;y<QUEENS_COUNT;y++)
	Sprite16(12*x,12*y,16,TileData_OFF,Plane0,SPRT_OR);
	
	PortRestore();
	
	
	// Start Research
	
	clrscr();
	printf("Searching... \n");
	
	SolutionsCount=0;
	
	// Recursive Function
	ChoosePlace(0);
	
	printf("Solutions Found : %d \n",SolutionsCount);
	printf("Press any key... \n");
	GKeyFlush(); // Cancel keystrokes during searching
	ngetchx();
	
	SolutionsIndex=0;
	DrawScreen();
	
	for (;;)
	{
		int key = ngetchx();
		
		if (key==KEY_LEFT)
		{
			SolutionsIndex--;
			if (SolutionsIndex<0)
			SolutionsIndex=SolutionsCount-1;
			
			DrawScreen();
		}
		else if (key==KEY_RIGHT)
		{
			SolutionsIndex++;
			if (SolutionsIndex==SolutionsCount)
			SolutionsIndex=0;
			
			DrawScreen();
		}
		else if (key==KEY_ESC)
		break;
	}
	
	if (Plane0)
	free(Plane0);
	
}
